package com.edgec.browserbackend.browser.service.Impl;

import com.edgec.browserbackend.account.domain.Account;
import com.edgec.browserbackend.account.exception.AccountErrorCode;
import com.edgec.browserbackend.account.repository.AccountRepository;
import com.edgec.browserbackend.browser.ErrorCode.BrowserErrorCode;
import com.edgec.browserbackend.browser.domain.Group;
import com.edgec.browserbackend.browser.domain.IpResource;
import com.edgec.browserbackend.browser.domain.Shop;
import com.edgec.browserbackend.browser.domain.UserShop;
import com.edgec.browserbackend.browser.dto.FilterDto;
import com.edgec.browserbackend.browser.dto.ShopDto;
import com.edgec.browserbackend.browser.dto.ShopPageInfo;
import com.edgec.browserbackend.browser.dto.ShopPageResultDto;
import com.edgec.browserbackend.browser.repository.GroupRepository;
import com.edgec.browserbackend.browser.repository.IpResourceRepository;
import com.edgec.browserbackend.browser.repository.ShopRepository;
import com.edgec.browserbackend.browser.repository.UserShopRepository;
import com.edgec.browserbackend.browser.service.ShopService;
import com.edgec.browserbackend.common.commons.error.ClientRequestException;
import com.edgec.browserbackend.common.commons.error.ErrorCode;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional
public class ShopServiceImpl implements ShopService {

    private final Logger logger = LoggerFactory.getLogger(ShopServiceImpl.class);

    @Autowired
    ShopRepository shopRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    UserShopRepository userShopRepository;

    @Autowired
    GroupRepository groupRepository;

    @Autowired
    IpResourceRepository ipResourceRepository;

    @Override
    public String addShop(String username, ShopDto shopDto) {
        Account account = accountRepository.findByName(username);
        String id = null;
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        if (account.getPermission() < 4) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        if (account.getShopCount() >= 10000) {
            throw new ClientRequestException(AccountErrorCode.SHOPMAX);
        }
        UserShop us = userShopRepository.findByUsernameAndShopId(username, shopDto.getShopId());
        Group group = groupRepository.findById(shopDto.getGroup()).orElse(null);
        if (group == null) {
            throw new ClientRequestException(BrowserErrorCode.GROUPNOTEXIST);
        }
        try {
            shopDto.setOwner(username);
            Shop shop = new Shop();
            shop.of(shopDto);
            id = shopRepository.save(shop).getShopId();
            UserShop userShop = new UserShop();
            userShop.setUsername(username);
            userShop.setShopId(id);
            if (shopDto.getGroup() != null && us == null) {
                userShop.setGroupId(shopDto.getGroup());
            }
            userShopRepository.save(userShop);
        }catch (Exception e) {
            throw new ClientRequestException(BrowserErrorCode.INFORMATIONNOTCOMPELETE);
        }
        return id;
    }

    @Override
    public List<String> addShops(String username, List<Shop> shops) {
        return null;
    }

    @Override
    public String updateShop(String username, ShopDto shopDto) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        if (shopDto == null || StringUtils.isBlank(shopDto.getShopId())) {
            throw new ClientRequestException(AccountErrorCode.OTHERS);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopDto.getShopId());
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop_old = shopRepository.findById(shopDto.getShopId()).orElseGet(null);
        if (shop_old == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        try {
            shop_old = shop_old.of(shopDto);
            shopRepository.save(shop_old);
        }catch (Exception e) {
            logger.error("fail to update", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.INFORMATIONNOTCOMPELETE);
        }
        return shop_old.getShopId();
    }

    @Override
    public void deleteShop(String username, String shopId) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        if (shopId == null) {
            throw new ClientRequestException(AccountErrorCode.OTHERS);
        }
        logger.error(username, shopId);
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        if (shop.getIpId() != null && shop.getIp() != null) {
            unBindShop(username, shopId, shop.getIp());
        }
        boolean result = userShopRepository.deleteByUsernameAndShopId(username, shopId);
        if (result) {
            shopRepository.deleteById(shopId);
        } else throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
    }

    @Override
    public void bindShop(String username, String shopId, String ipAddr) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        if (shop.getIpId() != null && shop.getIp() != null) {
            unBindShop(username, shopId, shop.getIp());
        }
        IpResource ipResource = ipResourceRepository.findByAddrAndIsDeleted(ipAddr, false);
        if (ipResource == null)
            throw new ClientRequestException(BrowserErrorCode.IPNOTEXIST);
        try {
            shop.setIp(ipAddr);
            shop.setIpId(ipResource.getId());
            shop.setIpRegion(ipResource.getIpRegion());
            shopRepository.save(shop);
            List<String> history = ipResource.getBindHistory();
            history.add(shop.getShopId());
            ipResource.setShopId(shopId);
            if (shop.getShopName()!=null)
                ipResource.setShopName(shop.getShopName());
            ipResourceRepository.save(ipResource);
        } catch (Exception e) {
            logger.error("fail to bind shop and ip", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }
    }

    @Override
    public void unBindShop(String username, String shopId, String ipAddr) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        if (account.getPermission() < 4 || userShop == null) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        IpResource ipResource = ipResourceRepository.findByAddrAndIsDeleted(ipAddr, false);
        if (ipResource == null)
            throw new ClientRequestException(BrowserErrorCode.IPNOTEXIST);
        if (!ipAddr.equals(shop.getIp()))
            throw new ClientRequestException(BrowserErrorCode.IPNOTBINDTOSHOP);
        try {
            shop.setIp(null);
            shop.setIpId(null);
            shop.setIpRegion(null);
            ipResource.setShopId(null);
            ipResource.setShopName(null);
            shopRepository.save(shop);
        } catch (Exception e) {
            logger.error("fail to unbind", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }
    }

    @Override
    public void transferShop(String username, String shopId, String groupId) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        UserShop userShop = userShopRepository.findByUsernameAndShopId(username, shopId);
        Group group = groupRepository.findById(groupId).orElse(null);
        if (account.getPermission() < 4 || userShop == null || !group.getOwner().equals(username)) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Shop shop = shopRepository.findById(shopId).orElse(null);
        if (shop == null)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        try {
            userShop.setGroupId(groupId);
            userShopRepository.save(userShop);
        } catch (Exception e) {
            logger.error("fail to unbind", e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }

    }

    @Override
    public void assignShops(String username, List<String> shopIds, List<String> users) {
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        List<UserShop> userShops = userShopRepository.findByUsernameAndShopIdIn(username, shopIds);
        if (account.getPermission() < 8 || userShops == null || userShops.size() < 1) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        Pageable pageable = PageRequest.of(0, 100);
        List<Shop> shops = shopRepository.findByShopIdIn(shopIds, pageable).getContent();
        if (shops == null || shops.size() < 1)
            throw new ClientRequestException(BrowserErrorCode.SHOPNOTEXIST);
        List<Account> accounts = accountRepository.findByNameIn(users);
        if (accounts != null || accounts.size() != users.size())
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        accounts.stream().forEach(x -> {
            try {
                for (Shop shop:shops){
                    UserShop userShop1 = userShopRepository.findByUsernameAndShopId(x.getName(), shop.getShopId());
                    if (userShop1 != null)
                        return;
                    userShopRepository.save(new UserShop(x.getName(), shop.getShopId()));
                }
            } catch (Exception e) {
                logger.error("fail to assign", e.getMessage());
                throw  new ClientRequestException(BrowserErrorCode.UNKNOWN);
            }
        });
    }

    @Override
    public ShopPageResultDto getShopList(String username, String groupId, int page, int amout, FilterDto filterDto) {
        if (amout > 100)
            amout = 100;
        Pageable pageable = PageRequest.of(page, amout);
        Account account = accountRepository.findByName(username);
        if (account == null) {
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        }
        Group group = null;
        if (groupId != null)
            group = groupRepository.findById(groupId).orElse(null);
        if (group == null)
            throw new ClientRequestException(BrowserErrorCode.GROUPNOTEXIST);
        if (account.getPermission() < 8 || (group != null && group.getOwner() != null && !group.getOwner().equals(username))) {
            throw new ClientRequestException(AccountErrorCode.NOPERMISSION);
        }
        List<String> shopIds = new ArrayList<>();
        if (groupId.equals("-1")) {
            shopIds = userShopRepository.findByUsername(username).stream().
                    map(x -> x.getShopId()).collect(Collectors.toList());
        } else {
            shopIds = userShopRepository.findByUsernameAndGroupId(username, groupId).stream().
                    map(x -> x.getShopId()).collect(Collectors.toList());
        }
        logger.error("shopIds.size " + shopIds.size());
        Page<Shop> shops;
        if (filterDto != null && StringUtils.isNotBlank(filterDto.getIpRegion()))
            shops = shopRepository.findByShopIdInAndIpRegionLike(shopIds, filterDto.getIpRegion(), pageable);
        else if (filterDto != null && StringUtils.isNotBlank(filterDto.getShopAccount()))
            shops = shopRepository.findByShopIdInAndShopAccountLike(shopIds, filterDto.getShopAccount(), pageable);
        else if (filterDto != null && StringUtils.isNotBlank(filterDto.getShopName()))
            shops = shopRepository.findByShopIdInAndShopNameLike(shopIds, filterDto.getShopName(), pageable);
        else
            shops = shopRepository.findByShopIdIn(shopIds, pageable);
        if (shops == null || shops.getNumberOfElements() < 1)
            return new ShopPageResultDto();
        List<ShopDto> shopDtos = new ArrayList<>();
        logger.error("numberofelem" + shops.getNumberOfElements());
        shops.getContent().stream().forEach(x -> {
            IpResource ipResource = ipResourceRepository.findByAddrAndIsDeleted(x.getIp(), false);
            if (ipResource == null)
                ipResource = new IpResource();
            String group1 = userShopRepository.findByUsernameAndShopId(username, x.getShopId()).getGroupId();
            shopDtos.add(ShopDto.of(x, group1, ipResource));
        });
        Page<ShopDto> shopDtoPage = new PageImpl<>(shopDtos, pageable, shopDtos.size());
        ShopPageResultDto shopPageResultDto = new ShopPageResultDto();
        shopPageResultDto.setShopList(shopDtoPage.getContent());
        ShopPageInfo shopPageInfo = new ShopPageInfo();
        shopPageInfo.setCurrentPage(shopDtoPage.getPageable().getPageNumber());
        shopPageInfo.setTotalPages(shopDtoPage.getTotalPages());
        shopPageInfo.setTotalShops(shopDtos.size());
        shopPageResultDto.setShopPage(shopPageInfo);
        return shopPageResultDto;
    }
}
